package net.dries007.holoInventory.network.request;

import io.netty.buffer.ByteBuf;
import net.dries007.holoInventory.Helper;
import net.dries007.holoInventory.network.response.PlainInventory;
import net.dries007.holoInventory.network.response.ResponseMessage;
import net.minecraft.block.Block;
import net.minecraft.block.BlockChest;
import net.minecraft.block.BlockJukebox;
import net.minecraft.inventory.IInventory;
import net.minecraft.inventory.InventoryBasic;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.tileentity.TileEntityChest;
import net.minecraft.tileentity.TileEntityEnderChest;
import net.minecraft.util.math.BlockPos;
import net.minecraft.world.World;
import net.minecraftforge.common.DimensionManager;
import net.minecraftforge.fml.common.network.simpleimpl.IMessageHandler;
import net.minecraftforge.fml.common.network.simpleimpl.MessageContext;
import net.minecraftforge.items.CapabilityItemHandler;

public class TileRequest extends RequestMessage
{
    private BlockPos pos;

    @SuppressWarnings("unused") // required for netty
    public TileRequest()
    {
        super();
    }

    public TileRequest(int dimension, BlockPos blockPos)
    {
        super(dimension);
        this.pos = blockPos;
    }

    @Override
    public void fromBytes(ByteBuf buf)
    {
        super.fromBytes(buf);
        pos = new BlockPos(buf.readInt(), buf.readInt(), buf.readInt());
    }

    @Override
    public void toBytes(ByteBuf buf)
    {
        super.toBytes(buf);
        buf.writeInt(pos.func_177958_n());
        buf.writeInt(pos.func_177956_o());
        buf.writeInt(pos.func_177952_p());
    }

    public static class Handler implements IMessageHandler<TileRequest, ResponseMessage>
    {
        @Override
        public ResponseMessage onMessage(TileRequest message, MessageContext ctx)
        {
            World world = DimensionManager.getWorld(message.dim);
            if (world == null) return null;
            TileEntity te = world.func_175625_s(message.pos);
            if (te == null) return null;

            if (Helper.banned.contains(te.getClass().getCanonicalName())) return null;

            if (te instanceof TileEntityEnderChest)
            {
                return new PlainInventory(message.pos, ctx.getServerHandler().field_147369_b.func_71005_bN());
            }
            else if (te instanceof BlockJukebox.TileEntityJukebox)
            {
                InventoryBasic ib = new InventoryBasic("minecraft:jukebox", false, 1);
                ib.func_70299_a(0, ((BlockJukebox.TileEntityJukebox) te).func_145856_a());
                return new PlainInventory(message.pos, ib).setName("tile.jukebox.name");
            }
            else if (te instanceof TileEntityChest)
            {
                Block b = world.func_180495_p(message.pos).func_177230_c();
                if (b instanceof BlockChest) return new PlainInventory(message.pos, ((BlockChest) b).func_180676_d(world, message.pos));
            }
            else if (te instanceof IInventory)
            {
                return new PlainInventory(message.pos, (IInventory) te);
            }
            else if (te.hasCapability(CapabilityItemHandler.ITEM_HANDLER_CAPABILITY, null))
            {
                return new PlainInventory(message.pos, te.func_145838_q().func_149739_a(), te.getCapability(CapabilityItemHandler.ITEM_HANDLER_CAPABILITY, null));
            }

            return null;
        }
    }
}
